#include<sys/types.h>
#include<sys/stat.h>
#include<unistd.h>
#include<dirent.h>
#include<stdlib.h>
#include<stdio.h>
#include "types.h"
#include "home.h"
#include "defaults.h"
#include "warning.h"
#include "mystring.h"
#include "X.h"
#include "window.h"
#include "keyboard.h"
#include "loop.h"
#include "edit.h"
#include "popup.h"
#include "save.h"
#include "load.h"
#include "output.h"
#include "gsl.h"

struct fileinfo{
  char *nazev;
  time_t mtime;
};

static int porovnej(struct fileinfo *a, struct fileinfo *b)
{
  if(a->mtime > b->mtime) return -1;
  if(a->mtime < b->mtime) return 1;
  return 0;
}

static DIR *adresar;

char **prectidir(int *pocet)
{
  struct dirent *de;
  struct stat st;
  int n, i, j, k, stare;
  char *fullname;
  struct fileinfo *files;
  char **vysledek;

  if(!homedir){
    warning("nebyl zjisten domovsky adresar");
    adresar = NULL;
    vysledek = (char **)malloc(sizeof(char *));
    vysledek[0] = NULL;
    *pocet = 0;
    return vysledek;
  }

  if((adresar = opendir(homedir)) == NULL){
    vysledek = (char **)malloc(sizeof(char *));
    vysledek[0] = NULL;
    *pocet = 0;
    return vysledek;
  }

  for(n = 0; readdir(adresar); n++);
  rewinddir(adresar);
  if(n) files = malloc(sizeof(struct fileinfo)*n);
  for(i = 0; (de = readdir(adresar));){
    j = strlen(de->d_name);
    k = strlen(SAVESUFFIX);
    while(j>0 && k>0 && de->d_name[--j] == SAVESUFFIX[--k]);
    if(k) continue;
    fullname = joinstrings(homedir, "/", de->d_name, NULL);
    stare = stat(fullname, &st);
    free(fullname);
    if(stare == -1) continue;
    de->d_name[j] = 0;
    files[i].nazev = de->d_name;
    files[i].mtime = st.st_mtime;
    i++;
  }
  n = i;
  qsort(files, n, sizeof(struct fileinfo),
	(int (*)(const void *, const void *))&porovnej);
  vysledek = (char **)malloc((n+1)*sizeof(char *));
  for(i=0; i<n; i++) vysledek[i] = files[i].nazev;
  vysledek[n] = NULL;
  free(files);

  *pocet = n;
  return vysledek;
}

static char filtrovano;

int filtruj(char **zdroj, char *str, char **cil)
{
  int i, j, k;

  if(zdroj != cil) filtrovano = 0;
  for(i=j=0; zdroj[i]; i++){
    for(k = 0; zdroj[i][k] && str[k] == zdroj[i][k]; k++);
    if(!str[k]) cil[j++] = zdroj[i];
    else filtrovano = 1;
  }
  cil[j] = NULL;
  return j;
}

void popup_sl(char *nadpis, void (*funkce)(char *))
{
  char **seznam;
  char **filznam;
  char *fullname;
  char ch, prepocitej;
  int pocet;
  char *otext, *str, ministr[2];
  char sltextpole[STRINGSIZE];

  int ogursor, odelka, ogelka, ocursor, oposunuti, ohist, ookr;

  seznam = prectidir(&pocet);
  filznam = (char **)malloc((pocet+1) * sizeof(char *));
  filtruj(seznam, "", filznam);

  zalohujtext(&ocursor, &odelka, &ogursor, &ogelka, &oposunuti);
  ohist = aktuhist;
  otext = text_pole;
  ookr = okrajtext;
  aktuhist = -1;
  text_pole = sltextpole;

  if(filznam[0]){
    if(okrajtext) okraj_to_win();
    zmentext(filznam[0]);
    aktuselect = 0;
  }
  else{
    if(!okrajtext) okraj_to_text();
    zmentext("");
    aktuselect = -1;
  }
  vytvorpsel(0.8, 0.8, 0.5, 0.5, nadpis, filznam, aktuselect, 0);

  ministr[1] = 0;

  XEvent report;

  for(;;){
    XNextEvent(display, &report);
    if(report.type == KeyPress){
      prepocitej = 0;
      if(report.xkey.keycode == KEY_ESC){
	if((okrajtext || filtrovano) && seznam[0]){
	  if(okrajtext) okraj_to_win();
	  aktuselect = 0;
	  popup_selpocet(filtruj(seznam, "", filznam));
	  zmentext(filznam[0]);
	  clearselect();
	  drawselect();
	}
	else{
	  str = NULL;
	  break;
	}
      }
      else if(report.xkey.keycode == KEY_ENTER){
	str = textcstr();
	if(str[0]) break;
	if(str != text_pole) free(str);
      }
      else if(report.xkey.keycode == KEY_TAB){
	if(okrajtext){
	  if(filznam[0]){
	    okraj_to_win();
	    aktuselect = 0;
	    oznacsel();
	    smazkursor();
	    prepocitej = 1;
	  }
	}
	else{
	  odoznacsel();
	  aktuselect = -1;
	  okraj_to_text();
	  ukazkursor();
	  prepocitej = 1;
	}
      }
      else if(okrajtext){
	     if(report.xkey.keycode == KEY_LEFT)  posunkursor(0);
	else if(report.xkey.keycode == KEY_RIGHT) posunkursor(1);
	else if(report.xkey.keycode == KEY_BACKSPACE) prepocitej = delete(1);
	else if(report.xkey.keycode == KEY_DELETE)    prepocitej = delete(0);
	else{
	  ch = xkeyznak(&report);
	  if(ch) prepocitej = pripisznak(ch);
	}
      }
      else{
	prepocitej = aktuselect;
	if(report.xkey.keycode == KEY_UP)         posunselect(-1);
	else if(report.xkey.keycode == KEY_DOWN)  posunselect(1);
	else if(report.xkey.keycode == KEY_LEFT)  posunselect(-2);
	else if(report.xkey.keycode == KEY_RIGHT) posunselect(2);
	else{
	  ch = xkeyznak(&report);
	  if(ch){
	    okraj_to_text();
	    odoznacsel();
	    aktuselect = -1;
	    ministr[0] = ch;
	    zmentext(ministr);
	  }
	}
	if(aktuselect == prepocitej) prepocitej = 0;
	else prepocitej = 1;
      }
      if(prepocitej){
	if(!okrajtext) zmentext(filznam[aktuselect]);
	else{
	  str = textcstr();
	  if(prepocitej > 0) popup_selpocet(filtruj(seznam, str, filznam));
	  else popup_selpocet(filtruj(filznam, str, filznam));
	  if(str != text_pole) free(str);
	  clearselect();
	  drawselect();
	}
      }
    }
    if(report.type == ButtonPress){
      str = NULL;
      break;
    }
    if(report.type == Expose){
      if(report.xexpose.window == textwin) redrawtext();
      else drawpopup(1);
    }
  }

  zruspsel();
  free(filznam);
  free(seznam);
  if(str){
    fullname = joinstrings(str, SAVESUFFIX, NULL);
    funkce(fullname);
    free(fullname);
  }
  if(str != text_pole) free(str);

  if(adresar) closedir(adresar);
  odzaltext(ocursor, odelka, ogursor, ogelka, oposunuti);
  aktuhist = ohist;
  text_pole = otext;
  if(ookr != okrajtext){
    if(ookr) okraj_to_text();
    else okraj_to_win();
  }
  cleartext();
  redrawtext();

  cekalose = 1;
}

void gsave()
{
  popup_sl(outputstring("_save", NULL), save_all);
}

void gload()
{
  popup_sl(outputstring("_load", NULL), load_all);
}
