#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<utime.h>
#include "warning.h"
#include "defaults.h"
#include "types.h"
#include "home.h"
#include "mystring.h"
#include "color.h"
#include "map.h"
#include "X.h"
#include "window.h"
#include "pixmap.h"
#include "widget.h"
#include "animwid.h"
#include "pin.h"
#include "loop.h"
#include "timer.h"
#include "post.h"
#include "autopost.h"
#include "pol.h"
#include "input.h"
#include "predmet.h"
#include "edit.h"
#include "languages.h"
#include "save.h"
#include "load.h"

//: cs

static char cs1[CSL], cs2[CSL+1];
static int cs1pos, cs2pos;

//: soubor

static FILE *soubor;
static char chyba;

//: byte

static unsigned char byte;
static char bpos;

//: load_start

void load_start (char *filename)
{
  int i;

  chyba = 0;
  soubor = homefopen(filename, "r");

  if(!soubor){
    chyba = 1;
    return;
  }
  bpos = 0;
  for(i = 0; i < CSL; i++) cs1[i] = 0;
  for(i = 0; i < CSL+1; i++) cs2[i] = 0;
  cs1pos = 0;
  cs2pos = 0;
}

//: load_end

void load_end (char *filename)
{
  char *fullname;

  if(!soubor) return;

  if(!chyba && fgetc(soubor) != EOF){
    warning("pozde ukonceny soubor");
    chyba = 1;
  }
  if(!chyba && (bpos && (unsigned char)(byte << (8-bpos)))){
    warning("nevyprazdneny posledni byte");
    chyba = 1;
  }
  if(!chyba && filename){
    fullname = joinstrings(homedir, "/", filename, NULL);
    if(utime(fullname, NULL) == -1) warning("nepodarilo se touchnout %s", fullname);
    free(fullname);
  }

  if(soubor) fclose(soubor);
}

//: check_idstr

void check_idstr ()
{
  int i;

  if(!identstring || chyba) return;
  for(i=0;; i++){
    if(load_char() != identstring[i]){
      warning("nesouhlasi identifikacni string");
      chyba = 1;
      return;
    }
    if(identstring[i] == 0) break;
  }
}

//: check_cs

void check_cs ()
{
  int i;

  if(chyba) return;

  for(i=0; i<CSL*2; i++) load_bit();
  for(i=0; i<CSL+1; i++)
    if(cs2[i] || (i < CSL && cs1[i])){
      chyba = 1;
      if(cs2[i]) warning("bad checksum cs2[%d]", i);
      else warning("bad checksum cs1[%d]", i);
      return;
    }
}

//: load_bit

char load_bit ()
{
  char vysledek;
  int ch;

  if(chyba) return 0;

  if(bpos == 0){
    bpos = 7;
    ch = fgetc(soubor);
    if(ch == EOF){
      chyba = 1;
      warning("prilis brzky konec souboru");
      return 0;
    }
    byte = ch;
  }
  else bpos--;

  vysledek = (JEDNA << bpos) & byte;
  if(vysledek) vysledek = 1;
  zapiscs(vysledek, cs1, &cs1pos, CSL);
  zapiscs(vysledek, cs2, &cs2pos, CSL+1);
  return vysledek;
}

//: load_trit

char load_trit ()
{
  char vysledek;

  vysledek = load_bit();
  if(vysledek){
    if(load_bit()) vysledek = -1;
  }
  return vysledek;
}

//: load_char

char load_char ()
{
  char pos, vysledek;

  vysledek = 0;
  for(pos = 7; pos >= 0; pos--)
    if(load_bit())
      vysledek |= JEDNA << pos;
  return vysledek;
}

//: load_unsint

unsigned int load_unsint ()
{
  int vysledek, mocnina;
  char dibit;

  vysledek = 0;

  for(mocnina = 1;; mocnina *= 3){
    dibit = 2*load_bit();
    dibit += load_bit();
    if(!dibit) break;
    vysledek += dibit*mocnina;
  }

  //  printf("   load_unsint(%d)\n", vysledek);

  return vysledek;
}

//: load_int

int load_int ()
{
  if(load_bit()) return -load_unsint()-1;
  else return load_unsint();
}

//: load_intoddo

int load_intoddo (int prvni, int posledni)
{
  int j;

  if(prvni > posledni){
    warning("interval <%d; %d> ma spatne usporadane hodnoty", prvni, posledni);
    j = prvni;
    prvni = posledni;
    posledni = j;
  }

  if(prvni == posledni) return prvni;

  j = (prvni + posledni + 1)/2;

  if(load_bit() == 0){
    return load_intoddo(prvni, j-1);
  }
  else{
    return load_intoddo(j, posledni);
  }
}

//: load_string

char *load_string ()
{
  unsigned int delka, i;
  char *vysledek;

  if(chyba) return NULL;

  delka = load_unsint();
  vysledek = (char *)malloc(sizeof(char)*(delka+1));
  if(vysledek){
    for(i=0; i<delka && !chyba; i++) vysledek[i] = load_char();
    vysledek[delka] = 0;
  }
  else{
    chyba = 1;
    warning("load: malloc vratil NULL");
  }

  return vysledek;
}

//: load_float

float load_float ()
{
  float vysledek;
  int j;

  j=1;
  vysledek = load_int();
  if(load_bit()) j *= 100;
  if(load_bit()) j *= 10;
  vysledek /= j;

  return vysledek;
}

//: loaded

static kq_pixmap *loaded_pixmaps;                                 // prvpix
static kq_widget *loaded_widgets, *loaded_hidwids;                // prvwidget, poshidwid
static kq_submap *loaded_submaps, *loaded_bgmap, *loaded_hidmaps; // prvsubmap, bgmap, prvhidsm
static kq_animpost *loaded_anips;                                 // prvanip
static kq_postava *loaded_posts;                                  // prvpost
static kq_timer *loaded_timers, *loaded_pautims;                  // prvtimer, pausedtimers
static kq_predmet *loaded_predmets;                               // prvpredmet
static kq_policko *loaded_pol;                                    // aktupol
static kq_anim *loaded_apole;                                     // animpole

//: pocty

static int pocet_pix, pocet_wid, pocet_hwid, pocet_sm, pocet_hsm,
  pocet_anip, pocet_post, pocet_tim, pocet_ptim, pocet_predm;

//: load_pocty

void load_pocty ()
{
  int i;
  kq_pixmap *ppom;
  kq_widget *wpom;
  kq_submap *smpom;
  kq_animpost *appom;
  kq_postava *popom;
  kq_timer *tpom;
  kq_predmet *prpom;

  loaded_pixmaps = NULL;
  loaded_widgets = NULL;
  loaded_hidwids = NULL;
  loaded_submaps = NULL;
  loaded_hidmaps = NULL;
  loaded_bgmap = NULL;
  loaded_anips = NULL;  
  loaded_posts = NULL;
  loaded_timers = NULL;
  loaded_pautims = NULL;
  loaded_predmets = NULL;
  loaded_apole = NULL;

  pocet_pix   =  load_unsint();
  pocet_wid   =  load_unsint();
  pocet_hwid  = -load_unsint();
  pocet_sm    =  load_unsint();
  pocet_hsm   = -load_unsint();
  pocet_anip  =  load_unsint();
  pocet_post  =  load_unsint();
  pocet_tim   =  load_unsint();
  pocet_ptim  = -load_unsint();
  pocet_predm =  load_unsint();

  loaded_pol = load_pol();

  if(chyba) return;

  if(loaded_pol){
    loaded_apole = malloc(sizeof(kq_anim)*loaded_pol->pocetanim);
    if(!loaded_apole){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    for(i=0; i<loaded_pol->pocetanim; i++){
      loaded_apole[i].pocet = load_unsint();
      loaded_apole[i].pix = NULL;
      loaded_apole[i].casy = NULL;
    }
  }
  
  check_cs();

  if(chyba) return;

  for(i = 0; i < pocet_pix; i++){
    ppom = (kq_pixmap *)malloc(sizeof(kq_pixmap));
    if(!ppom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    ppom->nazev = NULL;
    ppom->next = loaded_pixmaps;
    loaded_pixmaps = ppom;
  }

  for(i = 0; i < pocet_wid; i++){
    wpom = (kq_widget *)malloc(sizeof(kq_widget));
    if(!wpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    wpom->next = loaded_widgets;
    if(loaded_widgets) loaded_widgets->prev = wpom;
    loaded_widgets = wpom;
  }
  if(loaded_widgets) loaded_widgets->prev = NULL;

  for(i = 0; i > pocet_hwid; i--){
    wpom = (kq_widget *)malloc(sizeof(kq_widget));
    if(!wpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    wpom->prev = loaded_hidwids;
    if(loaded_hidwids) loaded_hidwids->next = wpom;
    loaded_hidwids = wpom;
  }
  if(loaded_hidwids) loaded_hidwids->next = NULL;

  for(i=0; i < pocet_sm; i++){
    smpom = (kq_submap *)malloc(sizeof(kq_submap));
    if(!smpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    smpom->nazev = NULL;
    smpom->next = loaded_submaps;
    loaded_submaps = smpom;
  }

  for(i=0; i > pocet_hsm; i--){
    smpom = (kq_submap *)malloc(sizeof(kq_submap));
    if(!smpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    smpom->nazev = NULL;
    smpom->next = loaded_hidmaps;
    loaded_hidmaps = smpom;
  }

  for(i=0; i < pocet_anip; i++){
    appom = (kq_animpost *)malloc(sizeof(kq_animpost));
    if(!appom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    appom->pix.pole = NULL;
    appom->next = loaded_anips;
    loaded_anips = appom;
  }

  for(i=0; i < pocet_post; i++){
    popom = (kq_postava *)malloc(sizeof(kq_postava));
    if(!popom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    popom->next = loaded_posts;
    loaded_posts = popom;
  }

  for(i=0; i < pocet_tim; i++){
    tpom = (kq_timer *)malloc(sizeof(kq_timer));
    if(!tpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    tpom->next = loaded_timers;
    loaded_timers = tpom;
  }

  for(i=0; i > pocet_ptim; i--){
    tpom = (kq_timer *)malloc(sizeof(kq_timer));
    if(!tpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    tpom->next = loaded_pautims;
    loaded_pautims = tpom;
  }

  for(i=0; i < pocet_predm; i++){
    prpom = (kq_predmet *)malloc(sizeof(kq_predmet));
    if(!prpom){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    prpom->nazev = NULL;
    prpom->next = loaded_predmets;
    loaded_predmets = prpom;
  }

  for(i=0; i<loaded_pol->pocetanim; i++){
    loaded_apole[i].pix = malloc(loaded_apole[i].pocet*sizeof(kq_pixmap *));
    if(!loaded_apole[i].pix){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
    loaded_apole[i].casy = malloc(loaded_apole[i].pocet*sizeof(unsigned int));
    if(!loaded_apole[i].casy){
      chyba = 1;
      warning("load: malloc vratil NULL");
      return;
    }
  }
}

//: load_color

kq_color *load_color ()
{
  unsigned int i, j;
  kq_color *vysledek;

  i = j = load_unsint();
  if(chyba || i == 0) return NULL;

  for(vysledek = loaded_pol->locbarva; vysledek && i>1;
      vysledek = vysledek->next) i--;

  if(!vysledek){
    chyba = 1;
    warning("prilis velke cislo pixmapy: %d", j);
  }
  return vysledek;
}

//: load_pointer

void *load_pointer (char typ)
{
  switch(typ){
  case SL_COLOR:  return load_color();
  case SL_PIXMAP: return load_pixmap();
  case SL_WIDGET: return load_widget();
  case SL_SUBMAP: return load_submap();
  case SL_ANIP:   return load_anip();
  case SL_POST:   return load_post();
  case SL_TIMER:  return load_timer();
  case SL_POL:    return load_pol();
  case SL_PREDM:  return load_predmet();
  case SL_WORD:   return load_word();
  case SL_AWID:   return load_awid();
  case SL_FUNC:   return load_func(NULL);
  default:
    warning("neznamy typ pointeru");
    chyba = 1;
  }
  return NULL;
}

//: load_func

void (*load_func (kq_pin *parametr))()
{
  sl_func *pom;
  int i;

  i = load_intoddo(0, pocet_func);

  if(i == 0 || chyba) return NULL;

  for(pom = prvslfunc; pom && i > 1; pom = pom->next) i--;

  if(!pom){
    warning("funkce nenalezena");
    chyba = 1;
    return NULL;
  }

  if(parametr){
    i = load_trit();
    if(i == 0) parametr->typ = PINNULL;
    else if(i < 0){
      parametr->typ = PININT;
      parametr->u.i = load_int();
    }
    else{
      parametr->typ = PINPOINTER;
      if(pom->typ >= SL_COLOR && pom->typ <= SL_FUNC)
	parametr->u.p = load_pointer(pom->typ);
      else{
	warning("funkce nema pridelen typ parametru");
	chyba = 1;
      }
    }
  }
  return pom->funkce;
}

//: load_vars

void load_vars ()
{
  sl_var *pom;

  for(pom = prvslvar; !chyba && pom; pom = pom->next){
    if(pom->typ >= SL_COLOR && pom->typ <= SL_FUNC)
      pom->loaded.p = load_pointer(pom->typ);
    else if(pom->typ == SL_BOOL) pom->loaded.ch = load_bit();
    else if(pom->typ == SL_TRIT) pom->loaded.ch = load_trit();
    else if(pom->typ == SL_CHAR) pom->loaded.ch = load_char();
    else if(pom->typ == SL_INT) pom->loaded.i = load_int();
    else if(pom->typ == SL_FLOAT) pom->loaded.f = load_float();
    else{
      warning("promenna nema prideleny typ");
      chyba = 1;
    }
  }
}

//: load_kontexts

void load_kontexts ()
{
  kq_word *pom;

  for(pom = prvulslovo; pom; pom = pom->next)
    pom->loaded_kontext = load_int();
}

//: load_word

kq_word *load_word ()
{
  kq_word *vysledek;
  int i;

  i = load_intoddo(0, pocet_word);

  if(i == 0 || chyba) return NULL;

  for(vysledek = prvulslovo; vysledek && i>1; vysledek = vysledek->next) i--;

  return vysledek;
}

//: load_pixmaps

void load_pixmaps ()
{
  kq_pixmap *pom;

  for(pom = loaded_pixmaps; !chyba && pom; pom = pom->next){
    pom->lokalni = load_bit();
    pom->nazev = load_string();
  }
}

//: load_pixmap

kq_pixmap *load_pixmap()
{
  int i;
  kq_pixmap *vysledek;

  i = load_intoddo(pocet_fixpix, pocet_pix);
  if(chyba || i == 0) return NULL;

  if(i > 0) for(vysledek = loaded_pixmaps; vysledek && i>1; vysledek = vysledek->next) i--;
  else for(vysledek = fixed_pixmaps; vysledek && i<-1; vysledek = vysledek->next) i++;

  return vysledek;
}

//: load_winfo

void load_winfo (kq_widget *w)
{
  w->obr = load_pixmap();
  w->x = load_int();
  w->y = load_int();
  w->z = load_float();
  w->lokalni = load_trit();
}

//: load_widgets

void load_widgets ()
{
  kq_widget *pom;

  for(pom = loaded_widgets; !chyba && pom; pom = pom->next) load_winfo(pom);
  for(pom = loaded_hidwids; !chyba && pom; pom = pom->prev) load_winfo(pom);
}

//: load_widget

kq_widget *load_widget ()
{
  kq_widget *vysledek;
  int i;

  i = load_intoddo(pocet_hwid, pocet_wid);

  if(i == 0 || chyba) return NULL;

  if(i > 0) for(vysledek = loaded_widgets; vysledek && i>1; vysledek = vysledek->next) i--;
  else for(vysledek = loaded_hidwids; vysledek && i<-1; vysledek = vysledek->prev) i++;

  return vysledek;
}

//: load_sminfo

void load_sminfo (kq_submap *sm)
{
  sm->nazev = load_string();
  sm->x = load_int();
  sm->y = load_int();
  sm->z = load_float();
  sm->lokalni = load_trit();
}

//: load_submaps

void load_submaps ()
{
  kq_submap *pom;

  loaded_bgmap = load_submap();
  for(pom = loaded_submaps; !chyba && pom; pom = pom->next) load_sminfo(pom);
  for(pom = loaded_hidmaps; !chyba && pom; pom = pom->next) load_sminfo(pom);
}

//: load_submap

kq_submap *load_submap ()
{
  kq_submap *vysledek;
  int i;

  i = load_intoddo(pocet_hsm, pocet_sm);

  if(i == 0 || chyba) return NULL;

  if(i > 0) for(vysledek = loaded_submaps; vysledek && i>1; vysledek = vysledek->next) i--;
  else for(vysledek = loaded_hidmaps; vysledek && i<-1; vysledek = vysledek->next) i++;

  return vysledek;
}

//: load_anips

void load_anips ()
{
  kq_animpost *pom;

  int pocetpix, i;

  for(pom = loaded_anips; !chyba && pom; pom = pom->next){
    pom->x        = load_int();
    pom->y        = load_int();
    pom->framu    = load_unsint();
    pom->tim      = load_unsint();
    pom->jednopol = load_trit();
    pom->porad    = load_bit();
    pom->lokalni  = load_bit();

    pocetpix = pom->framu;
    if(!pom->jednopol) pocetpix *= 4;
    if(pocetpix < 1){
      chyba = 1;
      warning("mene nez jedna (%d) pixmapa v animaci postavy", pocetpix);
    }
    else if(pocetpix == 1) pom->pix.map = load_pixmap();
    else{
      pom->pix.pole = (kq_pixmap **)malloc(pocetpix*sizeof(kq_pixmap *));
      if(!pom->pix.pole){
	chyba = 1;
	warning("load: malloc vratil NULL");
	return;
      }
      for(i = 0; i < pocetpix; i++) pom->pix.pole[i] = load_pixmap();
    }
  }
}

//: load_anip

kq_animpost *load_anip ()
{
  kq_animpost *vysledek;
  int i;

  i = load_intoddo(pocet_fixanip, pocet_anip);

  if(i == 0 || chyba) return NULL;

  if(i > 0) for(vysledek = loaded_anips; vysledek && i>1; vysledek = vysledek->next) i--;
  else for(vysledek = fixed_anips; vysledek && i<-1; vysledek = vysledek->next) i++;

  return vysledek;
}

//: load_posts

void load_posts ()
{
  kq_postava *pom;

  for(pom = loaded_posts; !chyba && pom; pom = pom->next){
    pom->x = load_int();
    pom->y = load_int();
    pom->xx = load_int();
    pom->yy = load_int();
    pom->tim = load_timer();
    pom->zdi = load_bit();
    pom->barvy = load_bit();
    pom->odraz = load_bit();
    pom->odejde = load_bit();

    pom->submap = load_submap();
    pom->sm_x = load_int();
    pom->sm_y = load_int();
    pom->chyt_vzdal = load_int();

    pom->chyt_func = load_func(NULL);
    pom->auto_func = load_func(NULL);
    pom->free_func = load_func(NULL);

    if(load_bit() == 0){
      pom->autou_typ = AUTOU_SOUR;
      pom->auto_u.sourad.x = load_int();
      pom->auto_u.sourad.y = load_int();
    }
    else{
      pom->autou_typ = AUTOU_POST;
      pom->auto_u.sledpost = load_post();
    }
    pom->auto_tim = load_timer();

    pom->animace = load_anip();
    pom->smer = load_unsint();
    pom->animstav = load_unsint();
    pom->animtim = load_timer();
    pom->xz = load_float();
    pom->yz = load_float();
    pom->z = load_float();
    pom->widget = load_widget();
    pom->lokalni = load_trit();
    pom->hide = load_bit();

    pom->zmena = load_bit();
  }
}

//: load_post

kq_postava *load_post ()
{
  kq_postava *vysledek;
  int i;

  i = load_intoddo(0, pocet_post);

  if(i == 0 || chyba) return NULL;

  for(vysledek = loaded_posts; vysledek && i>1; vysledek = vysledek->next) i--;

  return vysledek;
}

//: load_predmets

void load_predmets ()
{
  kq_predmet *pom;

  for(pom = loaded_predmets; !chyba && pom; pom = pom->next){
    pom->slovo = load_word();
    pom->nazev = load_string();
    pom->inv   = load_unsint();
    pom->stav  = load_bit();
    pom->post  = load_post();
    if(!pom->inv){
      pom->pol = load_pol();
      if(pom->pol){
	pom->pix_x = load_int();
	pom->pix_y = load_int();
	pom->pix_z = load_float();
	pom->sm_x  = load_int();
	pom->sm_y  = load_int();
	pom->sm_z  = load_float();
      }
    }
  }
}

//: load_predmet

kq_predmet *load_predmet ()
{
  kq_predmet *vysledek;
  int i;

  i = load_intoddo(0, pocet_predm);

  if(i == 0 || chyba) return NULL;

  for(vysledek = loaded_predmets; vysledek && i>1; vysledek = vysledek->next) i--;

  return vysledek;  
}

//: load_awids

void load_awids ()
{
  int i, j;

  for(i=0; i<loaded_pol->pocetanim; i++){
    loaded_apole[i].faze = load_intoddo(0, loaded_apole[i].pocet);
    loaded_apole[i].widget = load_widget();
    loaded_apole[i].timer = load_timer();
    for(j=0; j<loaded_apole[i].pocet; j++){
      loaded_apole[i].pix[j] = load_pixmap();
      loaded_apole[i].casy[j] = load_unsint();
    }
  }
}

//: load_awid

kq_anim *load_awid ()
{
  return &loaded_apole[load_intoddo(0, loaded_pol->pocetanim-1)];
}

//: load_tinfo

void load_tinfo (kq_timer *tim)
{
  tim->funkce = load_func(&tim->parametr);
  if(!chyba && !tim->funkce){
    warning("funkce timeru je NULL");
    chyba = 1;
    return;
  }
  tim->zbyva = load_unsint();
  tim->priorita = load_float();
  tim->opakovani = load_int();
  tim->cas = load_unsint();
  tim->pause = load_bit();
  tim->lokalni = load_trit();
}

//: load_timers

void load_timers ()
{
  kq_timer *pom;

  for(pom = loaded_timers; !chyba && pom; pom = pom->next) load_tinfo(pom);
  for(pom = loaded_pautims; !chyba && pom; pom = pom->next) load_tinfo(pom);  
}

//: load_timer

kq_timer *load_timer ()
{
  kq_timer *vysledek;
  int i;

  i = load_intoddo(pocet_ptim, pocet_tim);

  if(i == 0 || chyba) return NULL;

  if(i > 0) for(vysledek = loaded_timers; vysledek && i>1; vysledek = vysledek->next) i--;
  else for(vysledek = loaded_pautims; vysledek && i<-1; vysledek = vysledek->next) i++;

  return vysledek;  
}

//: load_pol

kq_policko *load_pol ()
{
  int i;
  kq_policko *vysledek;

  i = load_intoddo(0, pocet_pol);

  if(chyba || i == 0) return NULL;

  for(vysledek = prvpol; i>1 && vysledek; i--)
    vysledek = vysledek->next;

  return vysledek;
}

//: free_loaded

void free_loaded ()
{
  int i;
  kq_pixmap *ppom, *ppomo;
  kq_widget *wpom, *wpomo;
  kq_submap *smpom, *smpomo;
  kq_animpost *appom, *appomo;
  kq_postava *popom, *popomo;
  kq_timer *tpom, *tpomo;
  kq_predmet *prpom, *prpomo;

  for(ppom = loaded_pixmaps; ppom; ppom = ppomo){
    ppomo = ppom->next;
    if(ppom->nazev) free(ppom->nazev);
    free(ppom);
  }
  for(wpom = loaded_widgets; wpom; wpom = wpomo){
    wpomo = wpom->next;
    free(wpom);
  }
  for(wpom = loaded_hidwids; wpom; wpom = wpomo){
    wpomo = wpom->prev;
    free(wpom);
  }
  for(smpom = loaded_submaps; smpom; smpom = smpomo){
    smpomo = smpom->next;
    if(smpom->nazev) free(smpom->nazev);
    free(smpom);
  }
  for(smpom = loaded_hidmaps; smpom; smpom = smpomo){
    smpomo = smpom->next;
    if(smpom->nazev) free(smpom->nazev);
    free(smpom);
  }
  for(appom = loaded_anips; appom; appom = appomo){
    appomo = appom->next;
    if(appom->pix.pole &&
       (appom->framu > 1 || (appom->framu == 1 && !appom->jednopol)))
      free(appom->pix.pole);
    free(appom);
  }
  for(popom = loaded_posts; popom; popom = popomo){
    popomo = popom->next;
    free(popom);
  }
  for(tpom = loaded_timers; tpom; tpom = tpomo){
    tpomo = tpom->next;
    free(tpom);
  }
  for(tpom = loaded_pautims; tpom; tpom = tpomo){
    tpomo = tpom->next;
    free(tpom);
  }
  for(prpom = loaded_predmets; prpom; prpom = prpomo){
    prpomo = prpom->next;
    if(prpom->nazev) free(prpom->nazev);
    free(prpom);
  }
  if(loaded_apole){
    for(i=0; i<loaded_pol->pocetanim; i++){
      if(loaded_apole[i].pix) free(loaded_apole[i].pix);
      if(loaded_apole[i].casy) free(loaded_apole[i].casy);
    }
    free(loaded_apole);
  }
}

//: apply_load

void apply_load ()
{
  kq_pixmap *ppom;
  kq_widget *wpom;
  kq_submap *smpom;
  sl_var *vpom;
  kq_word *wopom;
  kq_predmet *prpom, **prpomo;

  // Promenne:

  for(vpom = prvslvar; vpom; vpom = vpom->next){
    if(vpom->typ == SL_CHAR || vpom->typ == SL_CHAR)
      *(char *)vpom->ukazatel = vpom->loaded.ch;
    else if(vpom->typ == SL_INT) *(int *)vpom->ukazatel = vpom->loaded.i;
    else if(vpom->typ == SL_FLOAT) *(int *)vpom->ukazatel = vpom->loaded.f;
    else *(void **)vpom->ukazatel = vpom->loaded.p;
  }

  // Kontexty slov:

  for(wopom = prvulslovo; wopom; wopom = wopom->next)
    wopom->kontext = wopom->loaded_kontext;

  // Pixmapy:

  deleteApixmaps();
  prvpix = loaded_pixmaps;
  for(ppom = prvpix; ppom; ppom = ppom->next){
    ppom->barvy = nactiXPM(ppom->nazev, &ppom->ibarvy,
			   &ppom->akanal, &ppom->ikanal);
    ppom->scaled = ratio;
  }

  // Policko:

  if(win) free_animpole();
  set_pol(loaded_pol);
  if(win) zmenokno(loaded_pol->jmpixmap);
  else vytvorokno(loaded_pol->jmpixmap);

  animpole = loaded_apole;


  // Widgety:

  deleteAwidgets();
  prvwidget = loaded_widgets;
  poshidwid = loaded_hidwids;
  if(prvwidget){
    for(wpom = prvwidget; wpom->next; wpom = wpom->next);
    for(; wpom; wpom = wpom->prev){
      wpom->hide = 0;
      if(wpom->obr){
	if(wpom->obr->scaled != ratio){
	  rescalePixmap(wpom->obr->ibarvy, &wpom->obr->barvy);
	  rescalePixmap(wpom->obr->ikanal, &wpom->obr->akanal);
	  wpom->obr->scaled = ratio;
	}
	wpom->okno = XCreateSimpleWindow(display, win, wpom->x*ratio, wpom->y*ratio,
					 wpom->obr->PIX_SIR*ratio,
					 wpom->obr->PIX_VYS*ratio, 0,
					 BlackPixel(display,screen),
					 WhitePixel(display,screen));
	XShapeCombineMask (display, wpom->okno, ShapeBounding,
			   0, 0, wpom->obr->akanal, ShapeSet);
	XSetWindowBackgroundPixmap(display, wpom->okno, wpom->obr->barvy);
	XMapWindow(display, wpom->okno);
      }
    }
  }
  for(wpom = poshidwid; wpom; wpom = wpom->next) wpom->hide = 1;

  mengraf = 1;

  // Submapy:

  autoreloadmap = 1;

  deleteAsubmaps();
  prvsubmap = loaded_submaps;
  prvhidsm = loaded_hidmaps;
  bgmap = loaded_bgmap;
  for(smpom = prvsubmap; smpom; smpom = smpom->next){
    smpom->map = nactimapu(smpom->nazev);
    smpom->hide = 0;
  }
  for(smpom = prvhidsm; smpom; smpom = smpom->next){
    smpom->map = nactimapu(smpom->nazev);
    smpom->hide = 1;
  }
  reloadmap();

  // Animace postav:

  deleteAanips();
  prvanip = loaded_anips;

  // Postavy:

  deleteAposts();
  prvpost = loaded_posts;
  menpost = 1;

  // Timery:

  deleteAtimers();
  prvtimer = loaded_timers;
  pausedtimers = loaded_pautims;

  //: Predmety:

  deleteApredmets();
  prvpredmet = loaded_predmets;
  for(prpom = prvpredmet; prpom; prpom = prpom->next){
    if(prpom->inv){
      prpomo = &inventar;
      while(*prpomo && prpom->inv < (*prpomo)->inv) prpomo = &(*prpomo)->polnext;
      prpom->polnext = *prpomo;
      *prpomo = prpom;
    }
    else if(prpom->pol){
      prpom->polnext = prpom->pol->predmety;
      prpom->pol->predmety = prpom;
    }
    if(prpom->slovo) prpom->slovo->informace.u.p = prpom;
  }
}

//: load_all

void load_all (char *filename)
{
  printf("LOAD: \%s/%s\n", homedir, filename);

  load_start(filename);
  if(!soubor){
    warning("soubor %s nemohl byt otevren pro cteni", filename);
    return;
  }
  check_idstr();
  load_pocty();
  load_pixmaps();
  load_widgets();
  load_submaps();
  load_anips();
  load_posts();
  load_timers();
  load_kontexts();
  load_predmets();
  load_awids();
  load_vars();
  check_cs();

  load_end(filename);
  if(chyba) free_loaded();
  else apply_load();
  if(chyba) warning("CHYBA PRI NACITANI");
}

//: load_conf

void load_conf ()
{
  int *l_delhist;
  char **l_hist;
  int l_poceth, l_aktuh, l_FPS, l_ratio, l_unfullratio;
  char l_okrajtext, l_fullscreened;
  char  *l_lang;
  kq_word *jazyk;

  int i, j;

  if(!SAVECONF){
    zmenFPS(0);
    return;
  }
  load_start(SAVECONF);

  if(!soubor){
    zmenFPS(0);
    return;
  }
  printf("LOAD_CONF: %s/%s\n", homedir, SAVECONF);

  l_poceth = load_unsint();
  l_aktuh = load_unsint();
  if(chyba){
    l_delhist = NULL;
    l_hist = NULL;
  }
  else{
    l_delhist = (int *)malloc((l_poceth+1)*sizeof(int));
    if(!l_delhist){
      l_hist = NULL;
      warning("malloc vratil NULL");
      chyba = 1;
    }
    else{
      l_hist = (char **)malloc((l_poceth+1)*sizeof(char *));
      if(!l_hist){
	warning("malloc vratil NULL");
	chyba = 1;
      }
    }
  }

  for(i=l_poceth; i >= 0 && !chyba; i--) l_delhist[i] = load_unsint();
  check_cs();
  if(l_hist){
    for(i=l_poceth; i >= 0; i--){
      if(chyba) l_hist[i] = NULL;
      else{
	l_hist[i] = (char *)malloc((l_delhist[i]+1)*sizeof(char));
	if(!l_hist[i]){
	  warning("malloc vratil NULL");
	  chyba = 1;
	}
      }
    }

    for(i=l_poceth; i >= 0 && !chyba; i--){
      for(j=0; j<l_delhist[i]; j++) l_hist[i][j] = load_char();
      l_hist[i][j] = 0;
    }
  }
  if(l_delhist) free(l_delhist);

  l_okrajtext = load_bit();
  l_ratio = load_unsint();
  l_fullscreened = load_bit();
  if(l_fullscreened) l_unfullratio = load_unsint();
  l_FPS = load_intoddo(0, maxFPS);
  l_lang = load_string();
  load_end(NULL);

  if(chyba){
    zmenFPS(0);
    if(l_hist){
      for(i=0; i<=l_poceth; i++)
	if(l_hist[i]) free(l_hist[i]);
    }
  }
  else{
    apply_loaded_hist(l_hist, l_poceth, l_aktuh);
    okrajtext = l_okrajtext;
    ratio = l_ratio;
    fullscreened = l_fullscreened;
    unfullratio = l_unfullratio;
    zmenFPS(l_FPS - aktuFPS);
    for(jazyk = prvlang; jazyk && strcmp(l_lang, jazyk->informace.u.p); jazyk = jazyk->next);
    if(jazyk) aktulang = jazyk;
    else printf("neznamy jazyk '%s'\n", l_lang);
  }
  if(l_hist) free(l_hist);
  if(l_lang) free(l_lang);
}
